@@ -31,22 +31,24 @@ func NewCmdExtension(f *cmdutil.Factory) *cobra.Command {
 			executable of the same name. All arguments passed to the %[1]sgh <extname>%[1]s invocation
 			will be forwarded to the %[1]sgh-<extname>%[1]s executable of the extension.
 
-			An extension cannot override any of the core gh commands.
+			An extension cannot override any of the core gh commands. If an extension name conflicts
+			with a core gh command you can use %[1]sgh extension exec <extname>%[1]s.
 
-			See the list of available extensions at <https://github.com/topics/gh-extension>
+			See the list of available extensions at <https://github.com/topics/gh-extension>.
 		`, "`"),
-		Aliases: []string{"extensions"},
+		Aliases: []string{"extensions", "ext"},
 	}
 
 	extCmd.AddCommand(
 		&cobra.Command{
-			Use:   "list",
-			Short: "List installed extension commands",
-			Args:  cobra.NoArgs,
+			Use:     "list",
+			Short:   "List installed extension commands",
+			Aliases: []string{"ls"},
+			Args:    cobra.NoArgs,
 			RunE: func(cmd *cobra.Command, args []string) error {
-				cmds := m.List(true)
+				cmds := m.List()
 				if len(cmds) == 0 {
-					return errors.New("no extensions installed")
+					return cmdutil.NewNoResultsError("no installed extensions found")
 				}
 				cs := io.ColorScheme()
 				t := utils.NewTablePrinter(io)
@@ -60,68 +62,89 @@ func NewCmdExtension(f *cmdutil.Factory) *cobra.Command {
 
 					t.AddField(fmt.Sprintf("gh %s", c.Name()), nil, nil)
 					t.AddField(repo, nil, nil)
-					var updateAvailable string
-					if c.UpdateAvailable() {
-						updateAvailable = "Upgrade available"
+					version := displayExtensionVersion(c, c.CurrentVersion())
+					if c.IsPinned() {
+						t.AddField(version, nil, cs.Cyan)
+					} else {
+						t.AddField(version, nil, nil)
 					}
-					t.AddField(updateAvailable, nil, cs.Green)
+
 					t.EndRow()
 				}
 				return t.Render()
 			},
 		},
-		&cobra.Command{
-			Use:   "install <repository>",
-			Short: "Install a gh extension from a repository",
-			Long: heredoc.Doc(`
-				Install a GitHub repository locally as a GitHub CLI extension.
-				
-				The repository argument can be specified in "owner/repo" format as well as a full URL.
-				The URL format is useful when the repository is not hosted on github.com.
-				
-				To install an extension in development from the current directory, use "." as the
-				value of the repository argument.
+		func() *cobra.Command {
+			var pinFlag string
+			cmd := &cobra.Command{
+				Use:   "install <repository>",
+				Short: "Install a gh extension from a repository",
+				Long: heredoc.Doc(`
+					Install a GitHub repository locally as a GitHub CLI extension.
 
-				See the list of available extensions at <https://github.com/topics/gh-extension>
-			`),
-			Example: heredoc.Doc(`
-				$ gh extension install owner/gh-extension
-				$ gh extension install https://git.example.com/owner/gh-extension
-				$ gh extension install .
-			`),
-			Args: cmdutil.MinimumArgs(1, "must specify a repository to install from"),
-			RunE: func(cmd *cobra.Command, args []string) error {
-				if args[0] == "." {
-					wd, err := os.Getwd()
+					The repository argument can be specified in "owner/repo" format as well as a full URL.
+					The URL format is useful when the repository is not hosted on github.com.
+
+					To install an extension in development from the current directory, use "." as the
+					value of the repository argument.
+
+					See the list of available extensions at <https://github.com/topics/gh-extension>.
+				`),
+				Example: heredoc.Doc(`
+					$ gh extension install owner/gh-extension
+					$ gh extension install https://git.example.com/owner/gh-extension
+					$ gh extension install .
+				`),
+				Args: cmdutil.MinimumArgs(1, "must specify a repository to install from"),
+				RunE: func(cmd *cobra.Command, args []string) error {
+					if args[0] == "." {
+						if pinFlag != "" {
+							return fmt.Errorf("local extensions cannot be pinned")
+						}
+						wd, err := os.Getwd()
+						if err != nil {
+							return err
+						}
+						return m.InstallLocal(wd)
+					}
+
+					repo, err := ghrepo.FromFullName(args[0])
 					if err != nil {
 						return err
 					}
-					return m.InstallLocal(wd)
-				}
 
-				repo, err := ghrepo.FromFullName(args[0])
-				if err != nil {
-					return err
-				}
+					if err := checkValidExtension(cmd.Root(), m, repo.RepoName()); err != nil {
+						return err
+					}
 
-				if err := checkValidExtension(cmd.Root(), m, repo.RepoName()); err != nil {
-					return err
-				}
-
-				if err := m.Install(repo); err != nil {
-					return err
-				}
-
-				if io.IsStdoutTTY() {
 					cs := io.ColorScheme()
-					fmt.Fprintf(io.Out, "%s Installed extension %s\n", cs.SuccessIcon(), args[0])
-				}
-				return nil
-			},
-		},
+					if err := m.Install(repo, pinFlag); err != nil {
+						if errors.Is(err, releaseNotFoundErr) {
+							return fmt.Errorf("%s Could not find a release of %s for %s",
+								cs.FailureIcon(), args[0], cs.Cyan(pinFlag))
+						} else if errors.Is(err, commitNotFoundErr) {
+							return fmt.Errorf("%s %s does not exist in %s",
+								cs.FailureIcon(), cs.Cyan(pinFlag), args[0])
+						}
+						return err
+					}
+
+					if io.IsStdoutTTY() {
+						fmt.Fprintf(io.Out, "%s Installed extension %s\n", cs.SuccessIcon(), args[0])
+						if pinFlag != "" {
+							fmt.Fprintf(io.Out, "%s Pinned extension at %s\n", cs.SuccessIcon(), cs.Cyan(pinFlag))
+						}
+					}
+					return nil
+				},
+			}
+			cmd.Flags().StringVar(&pinFlag, "pin", "", "pin extension to a release tag or commit ref")
+			return cmd
+		}(),
 		func() *cobra.Command {
 			var flagAll bool
 			var flagForce bool
+			var flagDryRun bool
 			cmd := &cobra.Command{
 				Use:   "upgrade {<name> | --all}",
 				Short: "Upgrade installed extensions",
@@ -142,23 +165,32 @@ func NewCmdExtension(f *cmdutil.Factory) *cobra.Command {
 					if len(args) > 0 {
 						name = normalizeExtensionSelector(args[0])
 					}
+					if flagDryRun {
+						m.EnableDryRunMode()
+					}
 					cs := io.ColorScheme()
 					err := m.Upgrade(name, flagForce)
 					if err != nil && !errors.Is(err, upToDateError) {
 						if name != "" {
 							fmt.Fprintf(io.ErrOut, "%s Failed upgrading extension %s: %s\n", cs.FailureIcon(), name, err)
+						} else if errors.Is(err, noExtensionsInstalledError) {
+							return cmdutil.NewNoResultsError("no installed extensions found")
 						} else {
 							fmt.Fprintf(io.ErrOut, "%s Failed upgrading extensions\n", cs.FailureIcon())
 						}
 						return cmdutil.SilentError
 					}
 					if io.IsStdoutTTY() {
+						successStr := "Successfully"
+						if flagDryRun {
+							successStr = "Would have"
+						}
 						if errors.Is(err, upToDateError) {
 							fmt.Fprintf(io.Out, "%s Extension already up to date\n", cs.SuccessIcon())
 						} else if name != "" {
-							fmt.Fprintf(io.Out, "%s Successfully upgraded extension %s\n", cs.SuccessIcon(), name)
+							fmt.Fprintf(io.Out, "%s %s upgraded extension %s\n", cs.SuccessIcon(), successStr, name)
 						} else {
-							fmt.Fprintf(io.Out, "%s Successfully upgraded extensions\n", cs.SuccessIcon())
+							fmt.Fprintf(io.Out, "%s %s upgraded extensions\n", cs.SuccessIcon(), successStr)
 						}
 					}
 					return nil
@@ -166,6 +198,7 @@ func NewCmdExtension(f *cmdutil.Factory) *cobra.Command {
 			}
 			cmd.Flags().BoolVar(&flagAll, "all", false, "Upgrade all extensions")
 			cmd.Flags().BoolVar(&flagForce, "force", false, "Force upgrade extension")
+			cmd.Flags().BoolVar(&flagDryRun, "dry-run", false, "Only display upgrades")
 			return cmd
 		}(),
 		&cobra.Command{
@@ -184,6 +217,31 @@ func NewCmdExtension(f *cmdutil.Factory) *cobra.Command {
 				return nil
 			},
 		},
+		&cobra.Command{
+			Use:   "exec <name> [args]",
+			Short: "Execute an installed extension",
+			Long: heredoc.Doc(`
+				Execute an extension using the short name. For example, if the extension repository is
+				"owner/gh-extension", you should pass "extension". You can use this command when
+				the short name conflicts with a core gh command.
+
+				All arguments after the extension name will be forwarded to the executable
+				of the extension.
+			`),
+			Example: heredoc.Doc(`
+				# execute a label extension instead of the core gh label command
+				$ gh extension exec label
+			`),
+			Args:               cobra.MinimumNArgs(1),
+			DisableFlagParsing: true,
+			RunE: func(cmd *cobra.Command, args []string) error {
+				if found, err := m.Dispatch(args, io.In, io.Out, io.ErrOut); !found {
+					return fmt.Errorf("extension %q not found", args[0])
+				} else {
+					return err
+				}
+			},
+		},
 		func() *cobra.Command {
 			promptCreate := func() (string, extensions.ExtTemplateType, error) {
 				var extName string
@@ -209,17 +267,17 @@ func NewCmdExtension(f *cmdutil.Factory) *cobra.Command {
 				Use:   "create [<name>]",
 				Short: "Create a new extension",
 				Example: heredoc.Doc(`
-				# Use interactively
-				gh extension create
+					# Use interactively
+					gh extension create
 
-				# Create a script-based extension
-				gh extension create foobar
+					# Create a script-based extension
+					gh extension create foobar
 
-				# Create a Go extension
-				gh extension create --precompiled=go foobar
+					# Create a Go extension
+					gh extension create --precompiled=go foobar
 
-				# Create a non-Go precompiled extension
-				gh extension create --precompiled=other foobar
+					# Create a non-Go precompiled extension
+					gh extension create --precompiled=other foobar
 				`),
 				Args: cobra.MaximumNArgs(1),
 				RunE: func(cmd *cobra.Command, args []string) error {
@@ -285,19 +343,19 @@ func NewCmdExtension(f *cmdutil.Factory) *cobra.Command {
 					}
 					link := "https://docs.github.com/github-cli/github-cli/creating-github-cli-extensions"
 					out := heredoc.Docf(`
-					%[1]s Created directory %[2]s
-					%[1]s Initialized git repository
-					%[1]s Set up extension scaffolding
-					%[6]s
-					%[2]s is ready for development!
+						%[1]s Created directory %[2]s
+						%[1]s Initialized git repository
+						%[1]s Set up extension scaffolding
+						%[6]s
+						%[2]s is ready for development!
 
-					%[4]s
-					%[5]s
-					- commit and use 'gh repo create' to share your extension with others
+						%[4]s
+						%[5]s
+						- commit and use 'gh repo create' to share your extension with others
 
-					For more information on writing extensions:
-					%[3]s
-				`, cs.SuccessIcon(), fullName, link, cs.Bold("Next Steps"), steps, goBinChecks)
+						For more information on writing extensions:
+						%[3]s
+					`, cs.SuccessIcon(), fullName, link, cs.Bold("Next Steps"), steps, goBinChecks)
 					fmt.Fprint(io.Out, out)
 					return nil
 				},
@@ -322,7 +380,7 @@ func checkValidExtension(rootCmd *cobra.Command, m extensions.ExtensionManager,
 		return fmt.Errorf("%q matches the name of a built-in command", commandName)
 	}
 
-	for _, ext := range m.List(false) {
+	for _, ext := range m.List() {
 		if ext.Name() == commandName {
 			return fmt.Errorf("there is already an installed extension that provides the %q command", commandName)
 		}
@@ -337,3 +395,10 @@ func normalizeExtensionSelector(n string) string {
 	}
 	return strings.TrimPrefix(n, "gh-")
 }
+
+func displayExtensionVersion(ext extensions.Extension, version string) string {
+	if !ext.IsBinary() && len(version) > 8 {
+		return version[:8]
+	}
+	return version
+}
